% Karakter statisztika készítő program map adattípussal
% Oláh Barbara

-module(kar_stat_map).

% Függvények exportálása
-export([karakter_statisztika/1, 
		 karakter_szama/2]).

% Ha egy üres listát kap a függveny, akkor egy üres map az eredmény
karakter_statisztika([])->
	% Új, üres map létrehozása
	maps:new();

% Ha nem üres a lista, akkor készítsük el a stasztikát
karakter_statisztika(Szoveg) when is_list(Szoveg)->
	% Új, üres map létrehozása
	Karakterek = maps:new(),
	% Statszitika készítő segédfüggvény meghívása
	% Paraméterek:
	% 	Karakterek: a statisztikát tartalmazó map
	%	kovetkezo_resz(Szoveg): egy tuple: {a következő karakter, a még vizsgálandó szöveg}
	stat(Karakterek, kovetkezo_resz(Szoveg));

% Minden más paraméter hibás
karakter_statisztika(_)->
	"Rossz parameter".

% A kapott szövegben megszámlálja a választott karaktert
karakter_szama(Karakter,Szoveg)->
	% A karakter statisztika elkészítése
	Map = karakter_statisztika(Szoveg),
	
	case maps:find(Karakter,Map) of
		{ok, Db} -> Db;		% Ha szerepel a statisztikában, akkor visszaadjuk a darabszámot
		_ -> 0				% Ha nem szerepel, akkor 0 a darabszám
	end.

% Statisztika készítő segédfüggvény
% Paraméterek:
%	Map: az eddig elkészült statisztika
%	{X, Y}: ahol X az éppen vizsgált karakter, Y a még vizsgálandó szöveg
stat(Map, {C,[]})->
	% Üres lista esetén végeztünk
	stasztikaba(Map, {C,[]});
stat(Map, {C,Szoveg})->
	% Betesszük a statisztikába a C karaktert
	UjMap = stasztikaba(Map, {C,[]}),
	% Folytatjuk a szöveg feldolgozását
	stat(UjMap,kovetkezo_resz(Szoveg)).

% Egy karakter betétele a statisztikába
% Visszatérési érték: a C karakterrel kibővített statisztika
stasztikaba(Map, {C,[]})->
	% Megnézzük, hogy benne van-e a karakter a map-ben
	case maps:is_key(C,Map) of
		% Ha igen, akkor lekérdezzük a darabszámot és azt eggyel növeljük
		true  -> Ertek = maps:get(C,Map),
				 maps:update(C,Ertek + 1,Map);
		% Ha nincs, akkor beletesszük 1-es darabszámmal
		false -> maps:put(C,1,Map)
	end.

% Léptetés a szövegben
% Visszatérési érték: {X, Y}, ahol X az aktuális karakter, Y a még vizsgálandó szöveg
kovetkezo_resz(Szoveg)->
	case Szoveg of
		[C] 	-> {[C], []};
		[H | T] -> {[H], T}
	end.
	
	