% Karakter statisztika készítő program proplist adattípussal
% Oláh Barbara

-module(kar_stat_prop).

% Függvények exportálása
-export([karakter_statisztika/1, 
		 karakter_szama/2]).
		 
% Ha egy üres listát kap a függveny, akkor egy üres lista az eredmény
karakter_statisztika([])->
	[];

% Ha nem üres a lista, akkor készítsük el a stasztikát
karakter_statisztika(Szoveg) when is_list(Szoveg)->
	% Statszitika készítő segédfüggvény meghívása
	% Paraméterek:
	% 	[]: az eddig elkészült statisztika
	%	kovetkezo_resz(Szoveg): egy tuple: {a következő karakter, a még vizsgálandó szöveg}
	stat([], kovetkezo_resz(Szoveg));

% Minden más paraméter hibás
karakter_statisztika(_)->
	"Rossz parameter".
	
% A kapott szövegben megszámlálja a választott karaktert	
karakter_szama(Karakter,Szoveg)->
	% A karakter statisztika elkészítése
	PropList = karakter_statisztika(Szoveg),
	case proplists:is_defined(Karakter,PropList) of
		true  -> proplists:get_value(Karakter,PropList); 	% Ha szerepel a statisztikában, akkor visszaadjuk a darabszámot
		false -> 0												% Ha nem szerepel, akkor 0 a darabszám
	end.

% Statisztika készítő segédfüggvény
% Paraméterek:
%	PropList: az eddig elkészült statisztika
%	{X, Y}: ahol X az éppen vizsgált karakter, Y a még vizsgálandó szöveg
stat(PropList, {C,[]})->
	% Üres lista esetén végeztünk
	stasztikaba(PropList, {C,[]});
stat(PropList, {C,Szoveg})->
	% Betesszük a statisztikába a C karaktert
	UjPropList = stasztikaba(PropList, {C,[]}),
	% Folytatjuk a szöveg feldolgozását
	stat(UjPropList,kovetkezo_resz(Szoveg)).

% Egy karakter betétele a statisztikába
% Visszatérési érték: a C karakterrel kibővített statisztika	
stasztikaba(PropList, {C,[]})->
	% Megnézzük, hogy benne van-e a karakter a listában
	case proplists:is_defined(C,PropList) of
		% Ha igen, akkor lekérdezzük a darabszámot és azt eggyel növeljük
		true  -> Ertek = proplists:get_value(C,PropList),
				 proplists:delete(C,PropList) ++ [{C,Ertek + 1}];
		% Ha nincs, akkor beletesszük 1-es darabszámmal
		false -> PropList ++ [{C,1}]
	end.

% Léptetés a szövegben
% Visszatérési érték: {X, Y}, ahol X az aktuális karakter, Y a még vizsgálandó szöveg	
kovetkezo_resz(Szoveg)->
	case Szoveg of
		[C] 	-> {[C], []};
		[H | T] -> {[H], T}
	end.